﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sparrow.IoC;

namespace Sparrow.Domain
{
    /// <summary>
    /// support for nested transaction context
    /// </summary>
    public class TransactionContext :IDisposable
    {
        public TransactionContext()
        {
            var transactionConext = ContextContainer.Current.Resolve<ITransactionContext>();
            if (transactionConext.IsStarted == false)
            {
                this.CurrentTransactionContext = transactionConext;
                this.CurrentTransactionContext.Begin();
            }
        }

        ITransactionContext CurrentTransactionContext
        {
            get;
            set;
        }


        public void Commit()
        {
            if (this.CurrentTransactionContext != null)
            {
                this.CurrentTransactionContext.Commit();
            }
        }

        public void Rollback()
        {
            if (this.CurrentTransactionContext != null)
            {
                this.CurrentTransactionContext.Rollback();
                this.CurrentTransactionContext = null;//free context
            }
        }

  
        #region Dispose


        ~TransactionContext()
        {
            Dispose(false); 
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                //free managed objects
                this.Rollback();
            }

            //free unmanaged objectss
         
            if (disposing)
            {
                //remove me from finalization list
                GC.SuppressFinalize(this);   
            }
        }

 
        public void Dispose()
        {
            Dispose(true);
        }

        #endregion
    }
}
