﻿using Sparrow.Globalization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Linq;
using System.Threading;
using System.IO;

namespace Sparrow.Tests
{

    using Sparrow.IoC;
    [TestClass()]
    public class GlobalizationTests
    {

        [TestCleanup]
        public void Cleanup()
        {
            var fileName = Environments.ApplicationPath + "\\I18N\\Elements.xml";
            if (File.Exists(fileName))
            {
                File.Delete(fileName);
            }
        }

        [TestMethod()]
        public void LocalizeEmpty()
        {
            string source = string.Empty; 
            string expected = string.Empty;
            string actual;
            actual = source.Localizable();
            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void LocalizeNull()
        {
            string source = null;
            string expected = string.Empty;
            string actual;
            actual = source.Localizable();
            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void LocalizeWhiteSpace()
        {
            string source = "   ";
            string expected = string.Empty;
            string actual;
            actual = source.Localizable();
            Assert.AreEqual(expected, actual);
        }

       // [TestMethod()]
        public void LocalizeSource()
        {
            string source = "source";
            source.Localizable();

            var globalProvider = ContextContainer.Shared.Resolve<IElementProvider>();
            globalProvider.Refresh(true);

            var provider = ContextContainer.Current.Resolve<IElementProvider>();

            Assert.IsNotNull(provider);

            var element = provider.Elements
                .Where(i => i.Name == source 
                    && i.Category == string.Empty 
                    && i.Culture == Thread.CurrentThread.CurrentCulture.Name
                    ).FirstOrDefault();

            Assert.IsNotNull(element);
            Assert.AreEqual(source, element.Value);
       
        }
    }
}
